#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include "common.h"
#include "change_tape.h"
#include "spantape.h"

#define ver "1.0"

int parse_blocksize(char *s) {
	int i;
	int mul = 1;
	debug("parse_blocksize: blocksize: %s", s);
	for (i = 0; i < strlen(s); i++) {
		if ((mul == 1) && isdigit(*(s+i))) continue;
		if (mul == 1)
			switch (*(s+i)) {
				case 'K': mul = 1024; break;
				case 'M': mul = 1024*1024; break;
				case 'k': mul = 1000; break;
				case 'm': mul = 1000*1000; break;
				default: error("Unknown unit (%c) in %s", *(s+i), s); break;
			}
		else
			error("Invalid character (%c) in %s", *(s+i), s); break;
	}
	debug("parse_blocksize: s: %d, mul: %d", atoi(s), mul);
	return atoi(s) * mul;
}
		
void help(char *s) {
	fprintf(stderr,
"Usage:	%s -r <device> [-options] [> out]  -  read backup from tapes\n"
"	%s -w <device> [-options] [< in]   - write backup to tapes\n"
"Options:\n"
"	-c <command>	command to execute when changing tapes. Should be in\n"
"			form suitable for system(). Substituted %%-entities:\n"
"			%%m (r for read, w for write), %%i (backup id),\n"
"			%%n (tape no), %%b (bytes read from/written to tape),\n"
"			%%B (bytes read/written total so far)\n"
"			Should return 0 on success, 1 on success & stop\n"
"			(only for read mode), anyting else on error\n"
"	-b <blocksize>	desired block size. Can use multiplicative suffixes:\n"
"			k (1000), K (1024), m (1000^2)	and M (1024^2)\n"
"	-n <max_tapes>	Maximum number of tapes to use\n"
"	-i <id>		backup ID. Default is to use random 16-digit hex number\n"
"	-f <filename>	use <filename> instead of stdin/stdout\n"
"	-T		don't use backup ID tags. Implies -E, excludes -i\n"
"	-E		don't write finish ID tag\n"
"	-F		run -c command before reading/writing first tape\n"
"	-L		run -c command after reading/writing last tape (%%n == 0)\n"
"	-d		set debug mode on\n"
"	-V		version info\n"
"	-h		this help\n"
	, s, s);
	exit(0);
}

void version() {
	fprintf(stderr, "\n"
"spantape - span single pipe across multiple SCSI tapes\n"
"version " ver " (c) 2005 Sebastian Zagrodzki <s.zagrodzki@net.icm.edu.pl>\n\n"
"This software is distributed under the GNU General Public License\n\n"
	);
	exit(0);
}

int main(int argc, char **argv) {

	char c;
	char *file = NULL;

	while ((c = getopt(argc, argv, "r:w:c:b:n:i:f:TEFLhdV")) != -1) {
		switch (c) {
			case 'r':
			case 'w':
				if (!mode) {
					mode = (c == 'r' ? MODE_READ : MODE_WRITE);
					tape_dev = optarg;
				} else
					error("Options -r and -w cannot be used together");
				break;
			case 'c':
				change_cmd = optarg; break;
			case 'b':
				blocksize = parse_blocksize(optarg);
				if (blocksize < 512)
					error("Blocksize must be >= 512");
				break;
			case 'n':
				max_tapes = atoi(optarg); break;
			case 'i':
				if (!tag_tapes)
					error("Options -i and -T cannot be used together");
				id = optarg; break;
			case 'f':
				file = optarg; break;
			case 'T':
				if (id)
					error("Options -i and -T cannot be used together");
				tag_tapes = 0; tag_end = 0; break;
			case 'E':
				tag_end = 0; break;
			case 'F':
				change_before_first = 1; break;
			case 'L':
				change_after_last = 1; break;
			case 'h':
				help(argv[0]); break;
			case 'd':
				debugging = 1; break;
			case 'V':
				version(); break;
			case '?':
				error("Nieznana opcja: -%c", optopt);
				break;
			default:
				error("getopt"); break;
		}
	}

	if (!mode) help(argv[0]);

	extfd = (mode == MODE_READ ? 1 : 0);
	if (file) {
		debug("Otwieram plik: %s", file);
		extfd = open(file, O_NOCTTY | (mode == MODE_READ ? O_WRONLY|O_CREAT|O_TRUNC : O_RDONLY));
		if (extfd < 0) { perror("open"); abort(); }
		debug("extfd: %d", extfd);
	}
		
	if (change_before_first) {
		debug("Zmiana tamy przed pierwsz operacj");
		change_tape();
	}
	
	notice("Beginning %s operation", (mode==MODE_READ ? "read" : "write"));
	start_time = time(NULL);
	if (mode == MODE_READ)
		do_read();
	else
		do_write();
	
	close(extfd);
	return 0;
}
