#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <time.h>

#include "common.h"
#include "readwrite.h"

block_stat read_block(char *buf, int fatal) {
// buf jest zaalokowane i rozmiaru blocksize

/* cytat z manuala:
 * W  trybie o zmiennej dugoci bloku ilo odczytywanych bajtw nie musi
 * dokadnie odpowiada dugoci bloku na tamie. Jeli liczba bajtw jest
 * wiksza  ni  nastpny blok na tamie, sterownik zwrci dane, a funkcja
 * zwrci rozmiar biecego bloku. Jeli dugo bloku  jest  wiksza  ni
 * liczba  bajtw,  zwrcona  zostanie  dana  ilo danych, poczwszy od
 * pocztku bloku, a reszta bloku zostanie odrzucona.
 * Uzupenienie: u mnie kiedy prbuj czyta mniej ni blocksize zwraca ENOMEM
 *
 * W trybie o staej dugoci bloku ilo odczytywanych  bajtw  moe by
 * dowolna,   gdy   wczone jest   buforowanie.   Musi natomiast  by
 * wielokrotnoci dugoci bloku, gdy buforowanie jest  wyczone.  
 */
	int r;

	block_stat result;

	if (drive_mode == 0) {
		// czytaj duymi porcjami. Jak sprbujesz wczyta z mao to
		// bdzie ENOMEM.
		while (1) {
			r = read(tape_fd, buf, blocksize);
			debug("read_block: read: %d", r);
			if (r < 0) {
				switch (errno) {
					case ENOMEM:
						error("Za may blocksize");
						break;
					case EINTR:
						break;
					default:
						if (fatal) {
							perror("read");
							abort();
						}
						result.stat = 0;
						result.len = 0;
						return result;
						break;
				}
			} else if (r == 0) {
				result.stat = 1; result.len = 0; return result;
			} else {
				result.stat = 1; result.len = r; return result;
			}
		}
	} else {
		// fixed blocksize
		// czytaj w takich porcjach jak ma ustawiony napd. Cao na
		// pewno jest w caych blokach, wic nie ma znaczenia ile
		// przeczytasz. Dla wygody dobieramy sobie do caych blokw.
		char *readptr;
		int toread;

		toread = blocksize;
		readptr = buf;

		while (1) {
			r = read(tape_fd, readptr, toread);
			if (r < 0) {
				if (errno != EINTR) {
					if (fatal) { perror("read"); abort(); }
					else {
						result.stat = 0;
						result.len = 0;
						break;
					}
				}
			} else if (r == 0) {
				result.stat = 1;
				if (toread != blocksize) {
					warning("naped w fixed blocksize zwrci kawaek bloku po czym powiedzia EOF");
					result.len = blocksize - toread;
				} else {
					result.len = 0;
				}
				return result;
			} else if (r == toread) { // doczyta do koca
				result.stat = 1;
				result.len = blocksize;
				return result;
			}
			toread -= r;
			readptr += r;
		}
	}
	error("read_block: wyskoczy poza gwne drzewo");
}

void read_tape() {
	block_stat b;
	char *blockbuf;
	int towrite, w;
	char *p;

	bytes_tape = 0;
	blocks_tape = 0;
	tape_start_time = time(NULL);
	if (!blocksize) error("BUG: blocksize == 0 w malloc()");
	blockbuf = (char*)malloc(blocksize);
	if (!blockbuf) { perror("malloc"); abort(); }
	// upewniamy si e terminal jest blocking
	if (fcntl(OUTFD, F_SETFL, 0) < 0) { perror("fcntl"); abort(); }
	while (1) {
		b = read_block(blockbuf, 1);
		
		if (b.len == 0) break; // EOF
		bytes_tape += b.len;
		bytes_total += b.len;
		blocks_tape++;
		blocks_total++;
		towrite = b.len;
		p = blockbuf;
		while (towrite > 0) {
			w = write(OUTFD, p, towrite);
			if (w < 0) { perror("write"); abort(); }
			// write zwraca 0 tylko w wypadku NONBLOCK
			towrite -= w;
			p += w;
		}
	}
	free(blockbuf);
}
		
int write_block(char *buf, int len) {
/* tutaj zawsze zapisujemy w takich kawakach jak blocksize. To znaczy te, e musimy dopeni dowolnymi mieciami kady kawaek, nawet jeli ma tylko par bajtw. My dopeniamy zerami. */
	// len jest co najwyej blocksize
	// retvals: 0 - ENOSPC, 1 - zapisane
	int w;
	int towrite;
	if ((drive_mode > 0)&&(blocksize > len)) {
		int i;
		debug("Fixed mode - dopelniam %d bajtw zerami", blocksize-len);
		for (i = 0; i < (blocksize - len); i++)
			*(buf + i) = '\0';
		towrite = blocksize;
	} else // jak by krtszy blok, to piszemy jak leci...
		towrite = len;
	while (1) {
		w = write(tape_fd, buf, towrite);
		debug("write_block: %d", w);
		if (w < 0) {
			if (errno == ENOSPC) return 0;
			else if (errno != EINTR) { perror("write"); abort(); }
		} else if (w == towrite) {
			return 1;
		} else
			error("BUG! Write zapisao tylko cz bloku. Co to znaczy?");
	}
}

char *write_tape(char *oldblock) {
	char *blockbuf;
	int r, toread;
	char *p;

	bytes_tape = 0;
	blocks_tape = 0;
	tape_start_time = time(NULL);

	if (!oldblock) {
		if (!blocksize) error("BUG: blocksize == 0 w malloc()");
		blockbuf = (char*)malloc(blocksize);
		if (!blockbuf) { perror("malloc"); abort(); }
		debug("Zaalokowany nowy bufor do pisania");
		toread = blocksize;
		p = blockbuf;
	} else {
		debug("Mam stary niezapisany bufor");
		toread = 0;
		blockbuf = oldblock;
		p = blockbuf + blocksize;
	}
	// upewniamy si e terminal jest blocking
	if (fcntl(INFD, F_SETFL, 0) < 0) { perror("fcntl"); abort(); }
	while (1) {
		while (toread) {
			r = read(INFD, p, toread);
			if (r == 0) // koniec wejcia
				break;
			else if (r < 0) {
				if (errno != EINTR) { perror("read"); abort(); }
			} else {
				p += r;
				toread -= r;
			}
		}
		if (toread == blocksize) // nic nie poszo - znaczy cakiem koniec
			return NULL;
		r = write_block(blockbuf, blocksize - toread);
		if (r == 0)
			return blockbuf;
		bytes_tape += blocksize - toread;
		bytes_total += blocksize - toread;
		blocks_tape++;
		blocks_total++;
		toread = blocksize;
		p = blockbuf;
	}
}

